const express = require('express');
const PujaCategoryMappingsDBApi = require('../../db/api/puja_category_mappings');
const wrapAsync = require('../helpers').wrapAsync;

const router = express.Router();   //creating the router

/**
 *  @swagger
 *  /api/puja_category_mappings:
 *    get:
 *      security:
 *        - bearerAuth: []
 *      tags: [PujaCategoryMappings]
 *      summary: Get all puja_category_mappings
 *      description: Get all puja_category_mappings
 *      responses:
 *        200:
 *          description: PujaCategoryMappings list successfully received
 *          content:
 *            application/json:
 *              schema:
 *                type: array
 *                items:
 *                  $ref: "#/components/schemas/PujaCategoryMappings"
 *        401:
 *          $ref: "#/components/responses/UnauthorizedError"
 *        404:
 *          description: Data not found
 *        500:
 *          description: Some server error
 */

router.get(
  '/',
  wrapAsync(async (req, res) => {
    const payload = await PujaCategoryMappingsDBApi.findAll(req.query);
    res.status(200).send(payload);
  }),
);    //get method to get the data of all puja_category_mappings

/**
 *  @swagger
 *  /api/puja_category_mappings/{id}:
 *    put:
 *      security:
 *        - bearerAuth: []
 *      tags: [PujaCategoryMappings]
 *      summary: Update the data of the selected item
 *      description: Update the data of the selected item
 *      parameters:
 *        - in: path
 *          name: id
 *          description: Item ID to update
 *          required: true
 *          schema:
 *            type: string
 *      requestBody:
 *        description: Set new item data
 *        required: true
 *        content:
 *          application/json:
 *            schema:
 *              properties:
 *                id:
 *                  description: ID of the updated item
 *                  type: string
 *                data:
 *                  description: Data of the updated item
 *                  type: object
 *                  $ref: "#/components/schemas/PujaCategoryMappings"
 *              required:
 *                - id
 *      responses:
 *        200:
 *          description: The item data was successfully updated
 *          content:
 *            application/json:
 *              schema:
 *                $ref: "#/components/schemas/PujaCategoryMappings"
 *        400:
 *          description: Invalid ID supplied
 *        401:
 *          $ref: "#/components/responses/UnauthorizedError"
 *        404:
 *          description: Item not found
 *        500:
 *          description: Some server error
 */

router.put(
  '/puja/:puja_id',
  wrapAsync(async (req, res) => {
    await PujaCategoryMappingsDBApi.updateByPujaId(req.params.puja_id, req.body.data, req.currentUser);
    const payload = true;
    res.status(200).send(payload);
  }),
);   //put method to update the data of the selected item

module.exports = router;