const express = require('express');
const getDbInstance = require('../../db/initialize');
const UsersDBApi = require('../../db/api/users');
const KarusersDBApi = require('../../db/api/karusers');
const Booking_participantsDBApi = require('../../db/api/booking_participants');
const PujarisDBApi = require('../../db/api/pujaris');
const Pujari_applicationsDBApi = require('../../db/api/pujari_applications');
const PujasDBApi = require('../../db/api/pujas');
const Puja_samagri_mappingsDBApi = require('../../db/api/puja_samagri_mappings');
const SamagriDBApi = require('../../db/api/samagri');
const Booking_samagri_mappingsDBApi = require('../../db/api/booking_samagri_mappings');
const BookingsDBApi = require('../../db/api/bookings');
const PujaProceduresDBApi = require('../../db/api/puja_procedures');
const PujaModelsDBApi = require('../../db/api/puja_models');
const Puja_models_procedure_mappingsDBApi = require('../../db/api/puja_models_procedure_mappings');
const Puja_models_samagri_mappingsDBApi = require('../../db/api/puja_models_samagri_mappings');
// const db = require('../../db/models');

const wrapAsync = require('../helpers').wrapAsync;

const router = express.Router();

/**
 *  @swagger
 *  /api/utility:
 *    get:
 *      security:
 *        - bearerAuth: []
 *      tags: [All Data]
 *      summary: Get all data
 *      description: Get all data
 *      responses:
 *        200:
 *          description: All data successfully received
 *          content:
 *            application/json:
 *              schema:
 *                type: array
 *                items:
 *                  $ref: '#/components/schemas/Pujas'
 *        401:
 *          $ref: '#/components/responses/UnauthorizedError'
 *        404:
 *          description: Data not found
 *        500:
 *          description: Some server error
 */

router.get(
  '/',
  wrapAsync(async (req, res) => {
    try {
      // Initialize database first and wait for it to complete
      const db = await getDbInstance();
      if (!db) {
        throw new Error('Database initialization failed');
      }

      // Helper function to safely get count
      const safeGetCount = async (name, apiCall) => {
        try {
          const result = await apiCall();
          // Handle cases where result might be undefined or not have expected structure
          if (!result) return 0;
          return result.count || result.data?.length || 0;
        } catch (error) {
          console.error(`Error getting ${name} count:`, error);
          return 0;
        }
      };

      // Get all counts
      const counts = {
        usersCount: await safeGetCount('users', () => UsersDBApi.findAll({})),
        karUsersCount: await safeGetCount('karUsers', () =>
          KarusersDBApi.findAll({})
        ),
        bookingApplicationsCount: await safeGetCount(
          'bookingApplications',
          () => Booking_participantsDBApi.findAll({})
        ),
        pujarisCount: await safeGetCount('pujaris', () =>
          PujarisDBApi.findAll({})
        ),
        pujariApplicationsCount: await safeGetCount('pujariApplications', () =>
          Pujari_applicationsDBApi.findAll({})
        ),
        pujasCount: await safeGetCount('pujas', () => PujasDBApi.findAll({})),
        pujaSamagriMappingsCount: await safeGetCount(
          'pujaSamagriMappings',
          () => Puja_samagri_mappingsDBApi.findAll({})
        ),
        samagriCount: await safeGetCount('samagri', () =>
          SamagriDBApi.findAll({})
        ),
        bookingSamagriCount: await safeGetCount('bookingSamagri', () =>
          Booking_samagri_mappingsDBApi.findAll({})
        ),
        bookingsCount: await safeGetCount('bookings', () =>
          BookingsDBApi.findAll({})
        ),
        proceduresCount: await safeGetCount('procedures', () =>
          PujaProceduresDBApi.findAll({})
        ),
        pujaModelsCount: await safeGetCount('pujaModels', () =>
          PujaModelsDBApi.findAll({})
        ),
        pujaModelProcedureMappingsCount: await safeGetCount(
          'pujaModelProcedureMappings',
          () => Puja_models_procedure_mappingsDBApi.findAll({})
        ),
        pujaModelSamagriMappingsCount: await safeGetCount(
          'pujaModelSamagriMappings',
          () => Puja_models_samagri_mappingsDBApi.findAll({})
        ),
      };

      res.status(200).send(counts);
    } catch (error) {
      console.error('Error in utility controller:', error);
      res.status(500).send({
        error: 'Internal server error',
        details: error.message,
      });
    }
  })
);

module.exports = router;
