const KarusersDBApi = require('../../db/api/karusers');
const ValidationError = require('./notifications/errors/validation');
const getDbInstance = require("../../db/initialize");

module.exports = class KarusersService {
  static async create(data, currentUser) {
    const db = await getDbInstance();
    const transaction = await db.sequelize.transaction();
    try {
      const createKarUserResponse = await KarusersDBApi.create(data, {
        currentUser,
        transaction,
      });

      await transaction.commit();
      return createKarUserResponse;
    } catch (error) {
      await transaction.rollback();
      throw error;
    }
  }

  static async update(data, id, currentUser) {
    const db = await getDbInstance();
    const transaction = await db.sequelize.transaction();
    try {
      let karusers = await KarusersDBApi.findBy({ id }, { transaction });

      if (!karusers) {
        throw new ValidationError('karusersNotFound');
      }

      await KarusersDBApi.update(id, data, {
        currentUser,
        transaction,
      });

      await transaction.commit();
      return karusers;
    } catch (error) {
      await transaction.rollback();
      throw error;
    }
  }
  
  static async remove(id, currentUser) {
    let transaction;
    const db = await getDbInstance();
    try {
      // Initialize DB if needed
      if (!db.sequelize) {
        await getDbInstance();
      }
      
      // Start transaction
      transaction = await db.sequelize.transaction();

      const result = await KarusersDBApi.remove(id, {
        currentUser: currentUser || { id: null },
        transaction
      });

      await transaction.commit();
      return result;
      
    } catch (error) {
      if (transaction) await transaction.rollback();
      throw new ValidationError(error.message);
    }
  }
};