import { ListEntities } from 'src/entities';
import { DataSource } from 'typeorm';

import dotenv from 'dotenv';
const env = process.env.NODE_ENV;
dotenv.config({ path: `./environments/.env.${env}` });

export const AppDataSource = new DataSource({
  type: 'mysql',
  driver: require('mysql2'),
  host: process.env.DB_HOST,
  port: Number(process.env.DB_PORT),
  username: process.env.DB_USERNAME,
  password: process.env.DB_PASSWORD,
  database: process.env.DB_DATABASE,
  entities: [...ListEntities],
});
export const transactionWrapper = (
  handler: (request: any) => Promise<Record<string, any>>
) => {
  const transactedHandler = async (event: any) => {
    const connection = AppDataSource;
    const queryRunner = connection.createQueryRunner();
    await queryRunner.connect();
    const entityManager = queryRunner.manager;
    await queryRunner.startTransaction();
    let res: any;
    try {
      res = await handler({ ...event, entityManager });
      if (res && res.statusCode !== 500) {
        await queryRunner.commitTransaction();
      } else {
        await queryRunner.rollbackTransaction();
      }
    } catch (error) {
      res = console.log({ ...error });
      await queryRunner.rollbackTransaction();
    } finally {
      await queryRunner.release();
    }
    return res;
  };
  return transactedHandler;
};

AppDataSource.initialize()
  .then(() => {
    console.log('Data Source has been initialized!');
  })
  .catch((err) => {
    console.error('Error during Data Source initialization', err);
  });
