const getDbInstance = require("../initialize");
const Sequelize = require("sequelize");
const Op = Sequelize.Op;
const { v4: uuid } = require("uuid");

module.exports = class Booking_samagri_mappingsDBApi {
  static async create(data, options) {
    const db = await getDbInstance();
    const currentUser = (options && options.currentUser) || { id: null };
    const transaction = (options && options.transaction) || undefined;

    const booking_samagri_mappings = await db.booking_samagri_mappings.create(
      {
        id: data.id || uuid(),

        booking_id: data.booking_id || null,
        samagri_id: data.samagri_id || null,
        no_of_standard_qty: data.no_of_standard_qty || null,
        importHash: data.importHash || null,
        createdById: currentUser.id,
        updatedById: currentUser.id,
      },
      { transaction }
    );

    return booking_samagri_mappings;
  }

  static async update(id, data, options) {
    const db = await getDbInstance();
    const currentUser = (options && options.currentUser) || { id: null };
    const transaction = (options && options.transaction) || undefined;

    const booking_samagri_mappings = await db.booking_samagri_mappings.findByPk(
      id,
      {
        transaction,
      }
    );

    await booking_samagri_mappings.update(
      {
        booking_id: data.booking_id || null,
        samagri_id: data.samagri_id || null,
        no_of_standard_qty: data.no_of_standard_qty || null,
        updatedById: currentUser.id,
      },
      { transaction }
    );

    return booking_samagri_mappings;
  }

  static async remove(id, options) {
    const db = await getDbInstance();
    const currentUser = (options && options.currentUser) || { id: null };
    const transaction = (options && options.transaction) || undefined;

    const booking_samagri_mappings = await db.booking_samagri_mappings.findByPk(
      id,
      options
    );

    await booking_samagri_mappings.update(
      {
        deletedBy: currentUser.id,
      },
      {
        transaction,
      }
    );

    await booking_samagri_mappings.destroy({
      transaction,
    });

    return booking_samagri_mappings;
  }

  static async findBy(where, options) {
    const db = await getDbInstance();
    const transaction = (options && options.transaction) || undefined;

    const booking_samagri_mappings = await db.booking_samagri_mappings.findOne(
      { where },
      { transaction }
    );

    if (!booking_samagri_mappings) {
      return booking_samagri_mappings;
    }

    const output = booking_samagri_mappings.get({ plain: true });

    return output;
  }

  static async findAll(filters = {}) {
    try {
      const db = await getDbInstance();

      // Extract pagination params
      const limit = Math.max(1, parseInt(filters.limit) || 20);
      const page = Math.max(1, parseInt(filters.page) || 1);
      const offset = (page - 1) * limit;

      // Define valid columns for filtering
      const validColumns = [
        "id",
        "booking_id",
        "samagri_id",
        "no_of_standard_qty",
        "isProvidedBy",
        "samagri_type",
        "createdAt",
        "updatedAt",
      ];

      // Create where clause with only valid columns
      const where = {};
      Object.keys(filters).forEach((key) => {
        if (validColumns.includes(key)) {
          where[key] = filters[key];
        }
      });

      // Add deletedAt check if using soft deletes
      where.deletedAt = null;

      const { rows, count } = await db.booking_samagri_mappings.findAndCountAll(
        {
          where,
          limit,
          offset,
          raw: true,
          include: [
            {
              model: db.samagri,
              required: false,
            },
          ],
        }
      );

      return {
        rows,
        count,
        limit,
        offset,
        page,
      };
    } catch (error) {
      console.error("Error in BookingSamagriMapping.findAll:", error);
      throw error;
    }
  }

  static async findAllAutocomplete(query, limit) {
    const db = await getDbInstance();
    let where = {};

    if (query) {
      where = {
        [Op.or]: [
          { id: query },
          {
            booking_id: {
              [Op.like]: `%${query}%`,
            },
          },
        ],
      };
    }

    const records = await db.booking_samagri_mappings.findAll({
      attributes: ["id", "booking_id"],
      where,
      limit: limit ? Number(limit) : undefined,
      order: [["booking_id", "ASC"]],
    });

    return records.map((record) => ({
      id: record.id,
      label: record.booking_id?.toString() || record.id,
    }));
  }
};
