/* eslint-disable @typescript-eslint/no-unused-vars */
const db = require("../models");
const getDbInstance = require("../initialize");
const FileDBApi = require("./file");
const crypto = require("crypto");
const Utils = require("../utils");
const { v4: uuid } = require("uuid");
const Sequelize = require("sequelize");
const Op = Sequelize.Op;
const { models } = require("../models/index");

module.exports = class BookingsDBApi {
  static async create(data, options) {
    const db = await getDbInstance();
    const currentUser = (options && options.currentUser) || { id: null };
    const transaction = (options && options.transaction) || undefined;

    if (!data.model) {
      console.warn("model_id is undefined."); //if model is not defined, then return error
    }

    const bookings = await db.bookings.create(
      {
        //id: data.id || null,
        model: data.model || null,
        user_id: data.user_id || null,
        pujari_id: data.pujari_id || null,
        puja_id: data.puja_id || null,
        pujari_karishye_notes: data.pujari_karishye_notes || null,
        price: data.price || null,
        event_type: data.event_type || null,
        address: data.address || null,
        start_date: data.start_date || null,
        end_date: data.end_date || null,
        duration_hrs: data.duration_hrs || null,
        final_price: data.final_price || null,
        booking_status: data.booking_status || null,
        booking_selling_price: data.booking_selling_price || null,
        pujari_cost: data.pujari_cost || null,
        importHash: data.importHash || null,
        createdById: currentUser.id,
        updatedById: currentUser.id,
        gst: data.gst || null,
        muhurtam: data.muhurtam || null,
        service_fee: data.service_fee || null,
        no_of_pujaris: data.no_of_pujaris || null,
      },
      { transaction }
    );

    const pujaSamagri = await db.puja_samagri_mappings.findAll(
      {
        where: {
          puja_id: data.puja_id,
        },
        attributes: [
          "samagri_id",
          "no_of_standard_qty",
          "importHash",
          "samagri_type",
          "isProvidedBy",
        ],
      },
      { transaction }
    );
    const samagriData = pujaSamagri.map((each) => {
      return {
        id: uuid(),
        booking_id: bookings.id || null,
        samagri_id: each.samagri_id || null,
        no_of_standard_qty: each.no_of_standard_qty || null,
        importHash: each.importHash || null,
        isProvidedBy: each.isProvidedBy || null,
        // eslint-disable-next-line no-dupe-keys
        isProvidedBy: getProvidedBy(each.samagri_type, each.isProvidedBy),
        samagri_type: each.samagri_type || null,
        createdById: currentUser.id,
        updatedById: currentUser.id,
      };
    });

    await db.booking_samagri_mappings.bulkCreate(samagriData, { transaction });

    const pujaProcedure = await db.puja_model_procedure_mappings.findAll(
      {
        where: {
          model_id: data.model,
        },
        attributes: ["procedure_id", "order"],
      },
      { transaction }
    );

    const procedureData = pujaProcedure.map((each, index) => {
      return {
        booking_id: bookings.id || null,
        procedure_id: each.procedure_id || null,
        order: each.order || index + 1,
      };
    });

    await db.booking_procedure_mappings.bulkCreate(procedureData, {
      transaction,
    });

    return bookings;
  }

  static async update(id, data, options) {
    const db = await getDbInstance();
    const currentUser = (options && options.currentUser) || { id: null };
    const transaction = (options && options.transaction) || undefined;

    const bookings = await db.bookings.findOne(
      { where: { id: id } },
      { transaction }
    );

    console.log("=== Data was =====", data);
    await db.bookings.update(
      {
        user_id: data.user_id || null,
        pujari_id: data.pujari_id || null,
        puja_id: data.puja_id || null,
        pujari_karishye_notes: data.pujari_karishye_notes || null,
        price: data.price || null,
        model: data.model || null,
        event_type: data.event_type || null,
        address: data.address || null,
        start_date: data.start_date || null,
        end_date: data.end_date || null,
        duration_hrs: data.duration_hrs || null,
        base_price: data.base_price || null,
        final_price: data.final_price || null,
        booking_selling_price: data.booking_selling_price || null,
        pujari_cost: data.pujari_cost || null,
        booking_status: data.booking_status || null,
        updatedById: currentUser.id,
        gst: data.gst || null,
        muhurtam: data.muhurtam || null,
        service_fee: data.service_fee || null,
        no_of_pujaris: data.no_of_pujaris || null,
      },
      {
        where: {
          id: id,
        },
      },
      { transaction }
    );

    return bookings;
  }

  static async remove(id, options) {
    const db = await getDbInstance();
    const currentUser = (options && options.currentUser) || { id: null };
    const transaction = (options && options.transaction) || undefined;

    // const bookings = await db.bookings.findByPk(id, options);

    // await bookings.update(
    //   {
    //     deletedBy: currentUser.id,
    //   },
    //   {
    //     transaction,
    //   },
    // );

    // await bookings.destroy({
    //   transaction,
    // });
    const bookings = await db.bookings.destroy({
      where: { id: id },
      force: true,
    });

    const bookingSamagriMappings = await db.booking_samagri_mappings.destroy({
      where: { booking_id: id },
      force: true,
    });

    const bookingProcedureMappings =
      await db.booking_procedure_mappings.destroy({
        where: { booking_id: id },
        force: true,
      });

    return bookings;
  }

  static async findBy(where, options) {
    const db = await getDbInstance();
    const transaction = (options && options.transaction) || undefined;

    const bookings = await db.bookings.findOne({
      where,
      transaction,
      raw: true,
      nest: true, // Add this to properly nest the results
    });

    return bookings;
  }

  static async findAll(filters = {}) {
    const db= await getDbInstance();
    try {
      const db = await getDbInstance();
      if (!db) {
        throw new Error("Database not initialized");
      }

      const limit = Math.max(0, parseInt(filters.limit) || 20);
      const page = Math.max(1, parseInt(filters.page) || 1);
      const offset = Math.max(0, (page - 1) * limit);

      // Create where clause by explicitly mapping valid column names
      const where = {};
      const validColumns = [
        "id",
        "user_id",
        "pujari_id",
        "puja_id",
        "model",
        "booking_status",
        "state",
        "payment_status",
        "samagri_status",
      ];

      Object.keys(filters).forEach((key) => {
        if (validColumns.includes(key)) {
          where[key] = filters[key];
        }
      });

      const { rows, count } = await db.bookings.findAndCountAll({
        where,
        limit,
        offset,
        order: [["createdAt", "DESC"]],
        raw: true,
        nest: true,
      });

      return {
        rows,
        count,
        limit,
        offset,
        page,
      };
    } catch (error) {
      console.error("Error in Booking.findAll:", error);
      throw error;
    }
  }

  static async findAllAutocomplete(query, limit) {
    const db= await getDbInstance();
    let where = {};

    if (query) {
      where = {
        [Op.or]: [
          { id: query }, // Direct match
          {
            id: {
              [Op.like]: `%${query}%`, // Use Op.like instead of Op.iLike for better compatibility
            },
          },
        ],
      };
    }

    const records = await db.bookings.findAll({
      attributes: ["id"],
      where,
      limit: limit ? Number(limit) : undefined,
      order: [["id", "ASC"]],
    });

    return records.map((record) => ({
      id: record.id,
      label: record.id,
    }));
  }

  static async findBookingProcedures(id) {
    const db= await getDbInstance();
      const { rows, count } =
        await db.booking_procedure_mappings.findAndCountAll({
          where: { booking_id: id },
          include: [
            {
              model: db.puja_procedures,
              as: "procedure",
              required: true,
              attributes: [
                "id",
                "title",
                "description",
                "language",
                "special_notes",
              ],
            },
          ],
          order: [["order", "ASC"]],
        });
      return { rows, count };
  }

  static async updateBookingProcedures(id, data) {
    const db = await getDbInstance();
    const transaction = await db.sequelize.transaction();
    try {
      await db.booking_procedure_mappings.destroy(
        {
          where: { booking_id: id },
        },
        {
          transaction,
        }
      );

      const procedureData = data.map((each) => {
        return {
          booking_id: each.booking_id || null,
          procedure_id: each.procedure_id || null,
          order: each.order || 0,
        };
      });

      await db.booking_procedure_mappings.bulkCreate(procedureData, {
        transaction,
      });

      await transaction.commit();
      return { status: "success" };
    } catch (error) {
      await transaction.rollback();
      throw error;
    }
  }

  static async updateBookingSamagri(id, data) {
    const db=await getDbInstance();
    const transaction = await db.sequelize.transaction();
    try {
      await db.booking_samagri_mappings.destroy(
        {
          where: { booking_id: id },
          force: true,
        },
        {
          transaction,
        }
      );

      console.log("===== data was =====", data);
      const samagriData = data.map((each) => {
        return {
          id: uuid(),
          booking_id: each?.booking_id || null,
          samagri_id: each?.samagri_id || null,
          no_of_standard_qty: each?.no_of_standard_qty || null,
          samagri_type: each?.samagri_type || null,
          isProvidedBy: each?.isProvidedBy || null,
          importHash: each.importHash || null,
        };
      });

      await db.booking_samagri_mappings.bulkCreate(samagriData, {
        transaction,
      });

      await transaction.commit();
      return { status: "success" };
    } catch (error) {
      await transaction.rollback();
      throw error;
    }
  }
};

const getProvidedBy = (samagriType, providedBy) => {
  if (samagriType == null) {
    return providedBy;
  }
  if (samagriType == "addon") {
    return "user";
  }
  return providedBy;
};
