const getDbInstance = require("../initialize");
const { v4: uuid } = require("uuid");

let db = null;

module.exports = class PujaCategoryMappingsDBApi {
  static async initialize() {
    if (!db) {
      db = await getDbInstance();
    }
  }

  static async create(data, options) {
    await this.initialize();
    const transaction = (options && options.transaction) || undefined;

    const pujaCategoryMapping = await db.puja_category_mappings.create(
      {
        id: data.id || uuid(),
        puja_id: data.puja_id,
        category_id: data.category_id,
        sub_category_id: data.sub_category_id,
      },
      { transaction }
    );

    return pujaCategoryMapping; //pujaCategoryMapping create function
  }

  static async findAll(filter, options) {
    //find method is for finding all the data from the database
    await this.initialize();
    const transaction = (options && options.transaction) || undefined;
    let where = {};

    if (filter) {
      if (filter.puja_id) {
        where = {
          ...where,
          puja_id: filter.puja_id,
        };
      }
      if (filter.category_id) {
        where = {
          ...where,
          category_id: filter.category_id,
        };
      }
      if (filter.sub_category_id) {
        where = {
          ...where,
          sub_category_id: filter.sub_category_id,
        };
      }
    }

    const pujaCategoryMappings = await db.puja_category_mappings.findAll({
      where,
      transaction,
    });

    return pujaCategoryMappings;
  }

  static async findBy(where, options) {
    //findBy method is to find woth the where and the options conditions
    await this.initialize();
    const transaction = (options && options.transaction) || undefined;

    const pujaCategoryMapping = await db.puja_category_mappings.findOne(
      { where },
      { transaction }
    );

    if (!pujaCategoryMapping) {
      return pujaCategoryMapping;
    }

    const output = pujaCategoryMapping.get({ plain: true });

    return output;
  }

  static async updateByPujaId(puja_id, data, options) {
    //updateByPujaId method is to update the data with the puja_id
    await this.initialize();
    const currentUser = (options && options.currentUser) || { id: null };
    const transaction = (options && options.transaction) || undefined;

    const pujaCategoryMapping = await db.puja_category_mappings.findOne({
      where: { puja_id },
      transaction,
    });

    if (!pujaCategoryMapping) {
      throw new Error("PujaCategoryMapping not found");
    }

    await pujaCategoryMapping.update(
      {
        category_id: data.category_id || null,
        sub_category_id: data.sub_category_id || null,
        updatedById: currentUser.id,
      },
      { transaction }
    );

    return pujaCategoryMapping;
  }
};
