import {
  Column,
  Entity,
  JoinColumn,
  ManyToOne,
  OneToOne,
  PrimaryColumn,
} from 'typeorm';
import { Bookings } from './bookings';
import { Samagri } from './samagri';

@Entity('booking_samagri_mappings')
export class BookingSamagriMappings {
  @PrimaryColumn({ name: 'id', type: 'varchar' })
  readonly id: string;

  @Column({ name: 'booking_id', type: 'int' })
  readonly bookingId: number;

  @Column({ name: 'samagri_id', type: 'varchar' })
  readonly samagriId: string;

  @Column({ name: 'no_of_standard_qty', type: 'int' })
  readonly qty: number;

  @Column({ name: 'samagri_type', type: 'varchar' })
  readonly samagriType: string;

  @Column({ name: 'isProvidedBy', type: 'varchar' })
  readonly isProvidedBy: string;

  @Column({ name: 'updatedAt', type: 'timestamp' })
  readonly updatedAt?: Date;

  @Column({ name: 'deletedAt', type: 'timestamp' })
  readonly deletedAt?: Date;

  @Column({ name: 'createdAt', type: 'timestamp' })
  readonly createdAt?: Date;

  @Column({ name: 'importHash', type: 'varchar' })
  readonly importHash?: string;

  @Column({ name: 'createdById', type: 'char' })
  readonly createdById?: string;

  @Column({ name: 'updatedById', type: 'char' })
  readonly updatedById?: string;

  @Column({ name: 'is_karishye_provided' })
  readonly isKarishyeProvided: string;

  @OneToOne(() => Samagri)
  @JoinColumn({ name: 'samagri_id', referencedColumnName: 'id' })
  readonly samagri!: Samagri;

  @ManyToOne(() => Bookings, (booking) => booking.bookingSamagriMappings)
  @JoinColumn({ name: 'booking_id' })
  readonly bookings!: Bookings;

  constructor(
    id: string,
    bookingId: number,
    samagriId: string,
    qty: number,
    isKarishyeProvided: string,
    samagriType: string,
    isProvidedBy: string,
    updatedAt?: Date,
    createdAt?: Date,
    deletedAt?: Date,
    importHash?: string,
    createdById?: string,
    updatedById?: string
  ) {
    this.id = id;
    this.bookingId = bookingId;
    this.samagriId = samagriId;
    this.qty = qty;
    this.isKarishyeProvided = isKarishyeProvided;
    (this.samagriType = samagriType),
    (this.isKarishyeProvided = isKarishyeProvided),
    (this.updatedAt = updatedAt);
    this.deletedAt = deletedAt;
    this.importHash = importHash;
    this.createdAt = createdAt;
    this.createdById = createdById;
    this.updatedById = updatedById;
  }
}
