import {
  Column,
  CreateDateColumn,
  Entity,
  JoinColumn,
  ManyToOne,
  OneToMany,
  OneToOne,
  PrimaryGeneratedColumn,
  UpdateDateColumn,
} from 'typeorm';
import { BookingParticipants } from './booking-participants';
import { BookingProcedureMappings } from './booking-procedure-mappings';
import { BookingSamagriMappings } from './booking-samagri-mappings';
import { BookingTransactions } from './booking-transactions';
import { KarishyeUsers } from './karishye-users';
import { PujaModels } from './puja-models';
import { Pujaris } from './pujaris';

@Entity('bookings')
export class Bookings {
  @PrimaryGeneratedColumn({ name: 'id', type: 'int' })
  readonly id: number;
  /**
   * added name and surname in the bookings table.
   */
  @Column({ name: 'name', type: 'varchar' })
  readonly name: string;

  @Column({ name: 'surname', type: 'varchar' })
  readonly surname: string;
  @Column({ name: 'user_id', type: 'varchar' })
  readonly userId?: string;

  @Column({ name: 'pujari_id', type: 'int' })
  readonly pujariId?: number;

  @Column({ name: 'model', type: 'int' })
  readonly model?: number;

  @Column({ name: 'pujari_karishye_notes', type: 'varchar' })
  readonly pujariKarishyeNotes?: string;

  @Column({ name: 'pujari_customer_notes' })
  readonly pujariCustomerNotes?: string;

  @Column({ name: 'customer_notes', type: 'varchar' })
  readonly customerNotes?: string;

  @Column({ name: 'event_type', type: 'varchar' })
  readonly eventType?: string;

  @Column({ name: 'address', type: 'varchar' })
  readonly address?: string;

  @Column({ name: 'address_latitude', type: 'double' })
  readonly addressLatitude?: number;

  @Column({ name: 'address_longitude', type: 'double' })
  readonly addressLongitude?: number;

  @Column({ name: 'booking_status' })
  readonly bookingStatus?: string;

  @Column({ name: 'state' })
  readonly state?: string;

  @Column({ name: 'samagri_status' })
  readonly samagriStatus?: string;

  @Column({ name: 'payment_status' })
  readonly paymentStatus?: string;

  @Column({ name: 'start_date', type: 'timestamp' })
  readonly startDate?: Date;

  @Column({ name: 'end_date', type: 'timestamp' })
  readonly endDate?: Date;

  @Column({ name: 'muhurtam', type: 'timestamp' })
  readonly muhurtam?: Date;

  @Column({ name: 'duration_hrs', type: 'int' })
  readonly durationHrs?: number;

  @Column({ name: 'no_of_pujaris', type: 'int' })
  readonly noOfPujaris?: number;

  @Column({ name: 'pujari_cost', type: 'int' })
  readonly pujariCost?: number;

  @Column({ name: 'booking_selling_price', type: 'int' })
  readonly bookingSellingPrice?: number;

  @Column({ name: 'gst', type: 'int' })
  readonly gst?: number;

  @Column({ name: 'service_fee', type: 'int' })
  readonly serviceFee?: number;

  @Column({ name: 'user_feedback', type: 'varchar' })
  readonly userFeedback?: string;

  @Column({ name: 'start_time', type: 'timestamp' })
  readonly startTime?: Date;

  @Column({ name: 'end_time', type: 'timestamp' })
  readonly endTime?: Date;

  @Column({ name: 'puja_id', type: 'int' })
  readonly pujaId?: number;

  @CreateDateColumn({
    type: 'timestamp',
    default: () => 'CURRENT_TIMESTAMP(6)',
  })
    createdAt?: Date;

  @UpdateDateColumn({
    type: 'timestamp',
    default: () => 'CURRENT_TIMESTAMP(6)',
    onUpdate: 'CURRENT_TIMESTAMP(6)',
  })
    updatedAt?: Date;

  @ManyToOne(() => KarishyeUsers, (userref) => userref.bookings)
  @JoinColumn({ name: 'user_id' })
  readonly karishye_users!: KarishyeUsers;

  @ManyToOne(() => Pujaris, (bookingref) => bookingref.bookings)
  @JoinColumn({ name: 'pujari_id' })
  readonly pujaris!: Pujaris;

  @ManyToOne(() => PujaModels, (modelref) => modelref.bookings)
  @JoinColumn({ name: 'model' })
  readonly pujamodels!: PujaModels;

  @OneToMany(
    () => BookingParticipants,
    (bookingpartici) => bookingpartici.booking
  )
  readonly bookingparticipants!: BookingParticipants;

  @OneToOne(() => PujaModels)
  @JoinColumn({ name: 'model', referencedColumnName: 'id' })
  readonly puja_model!: PujaModels;

  @OneToOne(() => KarishyeUsers)
  @JoinColumn({ name: 'user_id', referencedColumnName: 'id' })
  readonly user!: KarishyeUsers;

  @OneToMany(() => BookingTransactions, (transactions) => transactions.bookings)
  readonly bookingtransactions!: BookingTransactions;

  @OneToMany(
    () => BookingSamagriMappings,
    (samagrimapping) => samagrimapping.bookings
  )
  readonly bookingSamagriMappings!: BookingSamagriMappings;

  @OneToMany(
    () => BookingProcedureMappings,
    (proceduremapping) => proceduremapping.bookings
  )
  readonly bookingproceduremappings!: BookingProcedureMappings;

  constructor(
    id: number,
    name: string,
    surname: string,
    userId?: string,
    pujariId?: number,
    model?: number,
    pujariKarishyeNotes?: string,
    pujariCustomerNotes?: string,
    customerNotes?: string,
    eventType?: string,
    address?: string,
    addressLatitude?: number,
    addressLongitude?: number,
    bookingStatus?: string,
    state?: string,
    samagriStatus?: string,
    paymentStatus?: string,
    startDate?: Date,
    endDate?: Date,
    durationHrs?: number,
    noOfPujaris?: number,
    pujariCost?: number,
    bookingSellingPrice?: number,
    gst?: number,
    serviceFee?: number,
    userFeedback?: string,
    startTime?: Date,
    endTime?: Date,
    muhurtam?: Date,
    createdAt?: Date,
    updatedAt?: Date
  ) {
    this.id = id;
    this.userId = userId;
    this.name = name;
    this.surname = surname;
    this.pujariId = pujariId;
    this.model = model;
    this.pujariKarishyeNotes = pujariKarishyeNotes;
    this.pujariCustomerNotes = pujariCustomerNotes;
    this.customerNotes = customerNotes;
    this.eventType = eventType;
    this.address = address;
    this.addressLatitude = addressLatitude;
    this.addressLongitude = addressLongitude;
    this.bookingStatus = bookingStatus;
    this.gst = gst;
    this.state = state;
    this.samagriStatus = samagriStatus;
    this.paymentStatus = paymentStatus;
    this.startDate = startDate;
    this.endDate = endDate;
    this.durationHrs = durationHrs;
    this.noOfPujaris = noOfPujaris;
    this.pujariCost = pujariCost;
    this.serviceFee = serviceFee;
    this.bookingSellingPrice = bookingSellingPrice;
    this.userFeedback = userFeedback;
    this.startTime = startTime;
    this.endTime = endTime;
    this.muhurtam = muhurtam;
    this.createdAt = createdAt;
    this.updatedAt = updatedAt;
  }
}
