import {
  Column,
  Entity,
  JoinColumn,
  OneToMany,
  OneToOne,
  PrimaryColumn,
} from 'typeorm';
import { Bookings } from './bookings';
import { PujaModelProcedureMappings } from './puja-model-procedure-mappings';
import { PujaModelSamagriMappings } from './puja-model-samagri-mappings';
import { Pujas } from './pujas';

@Entity('puja_models')
export class PujaModels {
  @PrimaryColumn({ name: 'id', type: 'int' })
  readonly id: number;

  @Column({ name: 'puja_id', type: 'int' })
  readonly pujaId?: number;

  @Column({ name: 'duration_hrs', type: 'int' })
  readonly durationHrs?: number;

  @Column({ name: 'pujari_cost', type: 'int' })
  readonly pujariCost?: number;

  @Column({ name: 'no_of_pujaris', type: 'int' })
  readonly noOfPujaris?: number;

  @Column({ name: 'model_selling_price', type: 'int' })
  readonly modelSellingPrice?: number;

  @Column({ name: 'advance_amount', type: 'int' })
  readonly advanceAmount?: number;

  @Column({ name: 'is_popular_model', type: 'boolean' })
  readonly isPopularModel?: boolean;

  @Column({ name: 'updatedAt', type: 'timestamp' })
  readonly updatedAt?: Date;

  @Column({ name: 'deletedAt', type: 'timestamp' })
  readonly deletedAt?: Date;

  @Column({ name: 'createdAt', type: 'timestamp' })
  readonly createdAt?: Date;

  @Column({ name: 'importHash', type: 'varchar' })
  readonly importHash?: string;

  @Column({ name: 'createdById', type: 'char' })
  readonly createdById?: string;

  @Column({ name: 'updatedById', type: 'char' })
  readonly updatedById?: string;

  @Column({ name: 'indicative_cost_price', type: 'int' })
  readonly indicativeCostPrice?: number;

  @Column({ name: 'name', type: 'text' })
  readonly name?: string;

  @Column({ name: 'service_fee', type: 'int' })
  readonly serviceFee?: number;

  @Column({ name: 'samagri_cost', type: 'int' })
  readonly samagriCost?: number;

  @OneToMany(() => Bookings, (modelref) => modelref.pujamodels)
  readonly bookings!: Bookings;

  @OneToMany(() => PujaModelSamagriMappings, (modelref) => modelref.model)
  readonly samagris!: PujaModelSamagriMappings[];

  @OneToOne(() => Pujas)
  @JoinColumn({ name: 'puja_id', referencedColumnName: 'id' })
  readonly puja!: Pujas;

  @OneToMany(
    () => PujaModelProcedureMappings,
    (modelref) => modelref.pujamodels
  )
  readonly procedures!: PujaModelProcedureMappings[];

  constructor(
    id: number,
    pujaId?: number,
    durationHrs?: number,
    pujariCost?: number,
    noOfPujaris?: number,
    modelSellingPrice?: number,
    advanceAmount?: number,
    isPopularModel?: boolean,
    updatedAt?: Date,
    createdAt?: Date,
    deletedAt?: Date,
    importHash?: string,
    createdById?: string,
    updatedById?: string,
    indicativeCostPrice?: number,
    name?: string,
    serviceFee?: number,
    samagriCost?: number
  ) {
    this.id = id;
    this.pujaId = pujaId;
    this.durationHrs = durationHrs;
    this.noOfPujaris = noOfPujaris;
    this.modelSellingPrice = modelSellingPrice;
    this.advanceAmount = advanceAmount;
    this.isPopularModel = isPopularModel;
    this.pujariCost = pujariCost;
    this.indicativeCostPrice = indicativeCostPrice;
    this.name = name;
    this.updatedAt = updatedAt;
    this.deletedAt = deletedAt;
    this.importHash = importHash;
    this.createdAt = createdAt;
    this.createdById = createdById;
    this.updatedById = updatedById;
    this.serviceFee = serviceFee;
    this.samagriCost = samagriCost;
  }
}
