import { Column, Entity, PrimaryColumn, Timestamp } from 'typeorm';

@Entity('appointment')
export class Appointment {
  @PrimaryColumn({ name: 'appt_id', type: 'int' })
  readonly appointmentId: number;

  @Column({ name: 'user_id', type: 'varchar' })
  readonly userId: string;

  @Column({ name: 'patient_id', type: 'int' })
  readonly patientId: number;

  @Column({ name: 'appt_type_id', type: 'int' })
  readonly appointmentTypeId: number;

  @Column({ name: 'location_id', type: 'char' })
  readonly locationId: string;

  @Column({ name: 'appt_date', type: 'date' })
  readonly appointmentDate: string;

  @Column({ name: 'appt_time', type: 'time' })
  readonly appointmentTime: string;

  @Column({ name: 'appt_status', type: 'char' })
  readonly appointmentStatus?: string;

  @Column({ name: 'hipaa_sign_date', type: 'date' })
  readonly hipaaSignDate?: Date;

  @Column({ name: 'last_resched_date', type: 'date' })
  readonly lastReschedDate?: Date;

  @Column({ name: 'invoice_generated', type: 'int' })
  readonly invoiceGenerated?: number;

  @Column({ name: 'resched_count', type: 'int' })
  readonly reschedCount?: number;

  @Column({ name: 'appt_data', type: 'varchar' })
  readonly appointmentData?: string;

  @Column({ name: 'created_by', type: 'varchar' })
  readonly createdBy?: string;

  @Column({ name: 'appt_version', type: 'int' })
  readonly version?: number;

  @Column({ name: 'appt_history', type: 'varchar' })
  readonly history?: string;

  @Column({ name: 'create_time', type: 'timestamp' })
  readonly createTime?: Timestamp;

  @Column({ name: 'update_time', type: 'timestamp' })
  readonly updateTime?: Timestamp;

  @Column({ name: 'profile_update_info', type: 'varchar' })
  readonly profileUpdateInfo?: string;

  @Column({ name: 'invoice_raised_count', type: 'int' })
  readonly invoiceRaisedCount?: number;

  constructor(
    appointmentId: number,
    userId: string,
    patientId: number,
    invoiceRaisedCount: number,
    appointmentTypeId: number,
    locationId: string,
    appointmentDate: string,
    appointmentTime: string,
    appointmentStatus?: string,
    profileUpdateInfo?: string,
    hipaaSignDate?: Date,
    lastReschedDate?: Date,
    reschedCount?: number,
    appointmentData?: string,
    version?: number,
    history?: string,
    createTime?: Timestamp,
    updateTime?: Timestamp
  ) {
    this.appointmentId = appointmentId;
    this.invoiceRaisedCount = invoiceRaisedCount;
    this.userId = userId;
    this.patientId = patientId;
    this.appointmentTypeId = appointmentTypeId;
    this.locationId = locationId;
    this.appointmentDate = appointmentDate;
    this.appointmentTime = appointmentTime;
    this.profileUpdateInfo = profileUpdateInfo;
    this.appointmentStatus = appointmentStatus;
    this.hipaaSignDate = hipaaSignDate;
    this.lastReschedDate = lastReschedDate;
    this.reschedCount = reschedCount;
    this.appointmentData = appointmentData;
    this.version = version;
    this.history = history;
    this.createTime = createTime;
    this.updateTime = updateTime;
  }
}
