import { Injectable, Logger } from '@nestjs/common';
import { JwtService } from '@nestjs/jwt';
import { errorResponse, sendResponse } from 'src/common/common-utils';
import { AppDataSource } from 'src/common/db-utils';
import { KarishyeUsers } from 'src/entities';
import { Pujaris } from 'src/entities/pujaris';
import { TwilioService } from 'src/pujari/users/services/twilio/twilio.service';
import { UserPayload } from '../pujari/users/models/twilio.dto';
@Injectable()
export class LoginService {
  logger: Logger;

  constructor(private jwtService: JwtService) {
    this.logger = new Logger();
  }
  twilio: TwilioService = new TwilioService();

  async userLogin(data: UserPayload) {
    try {
      const userRepo = AppDataSource.getRepository(KarishyeUsers);
      const pujariRepo = AppDataSource.getRepository(Pujaris);

      let userInfo: any | null;
      if (data.isPujari === 'true') {
        userInfo = await pujariRepo.findOne({
          where: { phoneNumber: data.phoneNumber },
        });
        this.logger.log('userInfo', userInfo);
      } else {
        userInfo = await userRepo.findOne({
          where: { phoneNumber: data.phoneNumber },
        });
      }
      if (!userInfo) {
        throw { message: 'Mobile number not found', statusCode: 500 };
      }

      return sendResponse({ userInfo, optStatus: 200 });

      // if (userInfo) {
      //   return this.twilio
      //     .sendOTP(data)
      //     .then((optStatus: any) => {
      //       if (optStatus.statusCode === 500) {
      //         return errorResponse(optStatus);
      //       }
      //       return sendResponse({ optStatus });
      //     })
      //     .catch((err: any) => {
      // console.log("err",err);
      // return errorResponse(err);
      //     });
      // } else {
      //   throw { message: "Mobile number not found", statusCode: 500 };
      // }
    } catch (err) {
      this.logger.error('err', err);

      return errorResponse(err);
    }
  }

  async userSignUp(data: UserPayload) {
    try {
      const userRepo = AppDataSource.getRepository(KarishyeUsers);
      const pujariRepo = AppDataSource.getRepository(Pujaris);

      let userInfo: any | null;
      if (data.isPujari === 'true') {
        userInfo = await pujariRepo.findOne({
          where: { phoneNumber: data.phoneNumber },
        });
      } else {
        userInfo = await userRepo.findOne({
          where: { phoneNumber: data.phoneNumber },
        });
      }
      if (userInfo) {
        throw {
          message: 'User already exists with the mobile number',
          statusCode: 500,
        };
      }
      return sendResponse({ userInfo, optStatus: 200 });

      // return this.twilio
      //   .sendOTP(data)
      //   .then((optStatus: any) => {
      //     if (optStatus.statusCode === 500) {
      //       return errorResponse(optStatus);
      //     }
      //     return sendResponse({ optStatus });
      //   })
      //   .catch((err: any) => {
      //     return errorResponse(err);
      //   });
    } catch (err) {
      return errorResponse(err);
    }
  }

  async validateOTP(data: UserPayload) {
    try {
      let userInfo: any | null;

      if (data.isPujari === 'true') {
        userInfo = {
          id: 11,
          application_id: 11,
          name: 'Saikiran',
          surname: 'sashtri',
          date_of_birth: '1997-09-09',
          date_of_joining: '2022-02-27T13:00:00.000Z',
          qualification: 'BA',
          experience_yrs: 2,
          address: 'hyderabad',
          email_id: 'saikiran@divami.com',
          phone_number: '+918328573469',
          razorpay_id: 'test',
          gender: 'male',
          language: 'telugu',
          online_pujas: true,
          travel: true,
          city: null,
          is_active: true,
          importHash: null,
          createdAt: null,
          updatedAt: '2023-08-16T07:15:14.000Z',
          deletedAt: null,
          createdById: null,
          updatedById: '9',
          photo: [],
        };

        // const accessToken = this.jwtService.sign(payload)
        return sendResponse({ userInfo, optStatus: 200 });
      } else {
        userInfo = {
          id: '1d44a55e-6b5e-4888-8156-e019708549c3',
          name: 'PZK',
          surname: 'data',
          date_of_birth: '1992-07-22 00:00:00.000',
          date_of_registration: '2023-07-26 14:00:00',
          phone_number: '+919032790329',
          address_line1: 'Test the data with address lane 1',
          address_line2: 'Lane 2 address',
          city: 'Hyderabad',
          state: 'TS',
          country: 'India',
          pincode: '500045',
          gothram: 'Champion',
          nakshatram: 'Punarvasu',
          email_id: 'zameer@divami.com',
          gender: 'male',
          importHash: null,
          createdAt: '2023-07-27 08:52:11',
          updatedAt: '2023-07-27 10:40:11',
          deletedAt: null,
          createdById: '9',
          updatedById: '9',
        };
        return sendResponse({ userInfo, optStatus: 200 });
      }

      // if (data.isPujari === "true") {
      //   userInfo = await pujariRepo.findOne({
      //     where: { phoneNumber: data.phoneNumber },
      //   });
      // } else {
      //   userInfo = await userRepo.findOne({
      //     where: { phoneNumber: data.phoneNumber },
      //   });
      // }

      // if (
      //   1
      //   //userInfo
      // ) {
      //   return this.twilio
      //     .verifyOTP(data)
      //     .then((optStatus: any) => {
      //       if (optStatus.statusCode === 500) {
      //         return errorResponse(optStatus);
      //       }
      //       const payload = {
      //         name: userInfo.name,
      //         surname: userInfo.surname,
      //         dateOfBirth: userInfo.dateOfBirth,
      //         gothram: userInfo.gothram,
      //         emailId: userInfo.emailId,
      //         phoneNumber: userInfo.phoneNumber,
      //         gender: userInfo.gender,
      //         nakshatram: userInfo.nakshatram,

      //       };
      //     const accessToken = this.jwtService.sign(payload)
      //       return sendResponse({ accessToken, userInfo, optStatus });
      //     })
      //     .catch((err: any) => {
      //       return errorResponse(err);
      //     });
      // } else {
      //   throw { status: "Mobile number not found", statusCode: 500 };
      // }
    } catch (err) {
      return errorResponse(err);
    }
  }
}
