import { ApiProperty, ApiPropertyOptional } from '@nestjs/swagger';
import { Type } from 'class-transformer';
import {
  IsDate,
  IsEnum,
  IsNotEmpty,
  IsNumber,
  IsOptional,
  IsString,
  ValidateNested,
} from 'class-validator';
import {
  BOOKING_STATUS,
  EVENT_TYPE,
  PAYMENT_STATUS,
  SAMAGRI_STATUS,
  UpdateBookingStatus,
} from 'src/common/constants';

export class CreateBookingsDtoData {
  @ApiProperty()
  @IsNumber()
  @IsNotEmpty()
  public userId: string;
  /**
   * added name and surname in the payload.
   */
  @ApiProperty()
  @IsString()
  public name: string;

  @ApiProperty()
  @IsString()
  public surname: string;

  @ApiProperty()
  @IsNumber()
  @IsOptional()
  @IsNotEmpty()
  public id: number;

  @ApiPropertyOptional()
  @IsNumber()
  @IsOptional()
  public pujariId?: number;

  @ApiProperty()
  @IsNumber()
  public model: number;

  @ApiPropertyOptional()
  @IsString()
  @IsOptional()
  public pujariKarishyeNotes?: string;

  @ApiProperty()
  @IsString()
  public customerNotes: string;

  @ApiPropertyOptional()
  @IsString()
  @IsOptional()
  public pujariCustomerNotes?: string;

  @ApiProperty()
  @IsEnum(EVENT_TYPE)
  public eventType: EVENT_TYPE;

  @ApiProperty()
  @IsString()
  public address: string;

  @ApiPropertyOptional()
  @IsNumber()
  @IsOptional()
  public addressLatitude?: number;

  @ApiPropertyOptional()
  @IsNumber()
  @IsOptional()
  public addressLongitude?: number;

  @ApiPropertyOptional()
  @IsEnum(BOOKING_STATUS)
  @IsOptional()
  public bookingStatus?: BOOKING_STATUS;

  @ApiProperty()
  @IsString()
  public state: string;

  @ApiProperty()
  @IsEnum(SAMAGRI_STATUS)
  public samagriStatus?: SAMAGRI_STATUS;

  @ApiPropertyOptional()
  @IsEnum(PAYMENT_STATUS)
  @IsOptional()
  public paymentStatus?: PAYMENT_STATUS;

  @ApiProperty()
  @IsDate()
  @Type(() => Date)
  public startDate: Date;

  @ApiProperty()
  @IsDate()
  @Type(() => Date)
  public endDate: Date;

  @ApiPropertyOptional()
  @IsDate()
  @Type(() => Date)
  public muhurat?: Date;

  @ApiPropertyOptional()
  @IsNumber()
  public durationHrs: number;

  @ApiPropertyOptional()
  @IsNumber()
  public noOfPujaris: number;

  @ApiPropertyOptional()
  @IsNumber()
  public pujariCost: number;

  @ApiPropertyOptional()
  @IsNumber()
  public gst: number;

  @ApiProperty()
  @IsNumber()
  public serviceFee: number;

  @ApiProperty()
  @IsNumber()
  public bookingSellingPrice: number;

  @ApiProperty()
  @IsNumber()
  public pujaId: number;

  @ApiPropertyOptional()
  @IsString()
  public userFeedback: string;

  @ApiPropertyOptional()
  @IsDate()
  @Type(() => Date)
  public startTime: Date;

  @ApiPropertyOptional()
  @IsDate()
  @Type(() => Date)
  public endTime: Date;

  @ApiProperty()
  @IsDate()
  @Type(() => Date)
  public createdAt: Date;

  @ApiProperty()
  @IsDate()
  @Type(() => Date)
  public updatedAt: Date;
  // // Validates for a non-empty integer array
  // @IsArray()
  // @IsNumber({ allowNaN: false }, { each: true })
  // @ArrayMinSize(1)
  // public nums: number[];
}

export class EditBookingsDtoData {
  @ApiProperty()
  @IsNumber()
  @IsNotEmpty()
  public userId: string;

  @ApiProperty()
  @IsNumber()
  @IsOptional()
  @IsNotEmpty()
  public id: number;

  @ApiPropertyOptional()
  @IsNumber()
  @IsOptional()
  public pujariId?: number;

  @ApiPropertyOptional()
  @IsString()
  @IsOptional()
  public pujariKarishyeNotes?: string;

  @ApiProperty()
  @IsString()
  public customerNotes: string;

  @ApiPropertyOptional()
  @IsString()
  @IsOptional()
  public pujariCustomerNotes?: string;

  @ApiProperty()
  @IsEnum(EVENT_TYPE)
  public eventType: EVENT_TYPE;

  @ApiProperty()
  @IsString()
  public address: string;

  @ApiPropertyOptional()
  @IsNumber()
  @IsOptional()
  public addressLatitude?: number;

  @ApiPropertyOptional()
  @IsNumber()
  @IsOptional()
  public addressLongitude?: number;

  @ApiPropertyOptional()
  @IsEnum(BOOKING_STATUS)
  @IsOptional()
  public bookingStatus?: BOOKING_STATUS;

  @ApiProperty()
  @IsString()
  public state: string;

  @ApiProperty()
  @IsEnum(SAMAGRI_STATUS)
  public samagriStatus?: SAMAGRI_STATUS;

  @ApiPropertyOptional()
  @IsEnum(PAYMENT_STATUS)
  @IsOptional()
  public paymentStatus?: PAYMENT_STATUS;

  @ApiProperty()
  @IsDate()
  @Type(() => Date)
  public startDate: Date;

  @ApiProperty()
  @IsDate()
  @Type(() => Date)
  public endDate: Date;

  @ApiPropertyOptional()
  @IsDate()
  @Type(() => Date)
  public muhurtam?: Date;

  @ApiPropertyOptional()
  @IsNumber()
  public durationHrs: number;

  @ApiPropertyOptional()
  @IsNumber()
  public noOfPujaris: number;

  @ApiPropertyOptional()
  @IsNumber()
  public pujariCost: number;

  @ApiProperty()
  @IsNumber()
  public bookingSellingPrice: number;

  @ApiProperty()
  @IsNumber()
  public gst: number;

  @ApiProperty()
  @IsNumber()
  public serviceFee: number;

  @ApiProperty()
  @IsNumber()
  public pujaId: number;

  @ApiPropertyOptional()
  @IsString()
  public userFeedback: string;

  @ApiPropertyOptional()
  @IsDate()
  @Type(() => Date)
  public startTime: Date;

  @ApiPropertyOptional()
  @IsDate()
  @Type(() => Date)
  public endTime: Date;

  @ApiProperty()
  @IsDate()
  @Type(() => Date)
  public createdAt: Date;

  @ApiProperty()
  @IsDate()
  @Type(() => Date)
  public updatedAt: Date;
}

export class CreateBookingsDto {
  @ApiProperty()
  @IsString()
  public status: string;

  @ApiProperty({
    type: CreateBookingsDtoData,
    isArray: true,
  })
  @ValidateNested()
  @Type(() => CreateBookingsDtoData)
  public data: CreateBookingsDtoData;
}

export class EditBookingsDto {
  @ApiProperty({
    type: EditBookingsDtoData,
    isArray: true,
  })
  @ValidateNested()
  @Type(() => EditBookingsDtoData)
  public data: EditBookingsDtoData;
}

export class BookingSamagriMapping {
  @ApiProperty()
  @IsNumber()
  public samagriId: number;

  @ApiProperty()
  @IsNumber()
  public qty: number;

  @ApiProperty()
  @IsString()
  public isKarishyeProvided: string;
}
export class CreateBookingPayload {
  @ApiProperty({
    type: CreateBookingsDtoData,
  })
  public data: CreateBookingsDtoData;

  @ApiProperty({
    type: Number,
    isArray: true,
  })
  @IsNumber({}, { each: true })
  public procedures: number[];

  @ApiProperty({
    type: BookingSamagriMapping,
    isArray: true,
  })
  public samagri: BookingSamagriMapping[];
}

export class EditBookingPayload {
  @ApiProperty({
    type: EditBookingsDtoData,
  })
  public data: EditBookingsDtoData;

  @ApiProperty({
    type: Number,
    isArray: true,
  })
  @IsNumber({}, { each: true })
  public procedures: number[];

  @ApiProperty({
    type: BookingSamagriMapping,
    isArray: true,
  })
  public samagri: BookingSamagriMapping[];
}

export class AcceptBookingPayload {
  @ApiProperty({ type: Number })
  @IsNumber()
    id: number;

  @ApiProperty({ type: Number })
  @IsNumber()
    userId: number;
}

export class responseMessage {
  @ApiProperty({ type: String })
    status: string;

  @ApiProperty({ type: String })
    message: string;
}

export class UpdateBookingResponse {
  @ApiProperty()
  @IsString()
  public start_time: string;

  @ApiProperty()
  @IsString()
  public end_time: string;

  @ApiProperty({ type: Number })
  @IsNumber()
  public id: number;
}

export class UpdateBookingPayload extends UpdateBookingResponse {
  @ApiProperty()
  @IsEnum(UpdateBookingStatus)
    status: UpdateBookingStatus;

  @ApiProperty()
  @IsEnum(PAYMENT_STATUS)
  @IsOptional()
    paymentStatus: PAYMENT_STATUS;
}

export class UpdateBookingStatuses {
  @ApiProperty()
  @IsEnum(BOOKING_STATUS)
  @IsOptional()
    bookingStatus: BOOKING_STATUS;

  @ApiProperty()
  @IsEnum(PAYMENT_STATUS)
  @IsOptional()
    paymentStatus: PAYMENT_STATUS;

  @ApiProperty()
  @IsEnum(SAMAGRI_STATUS)
  @IsOptional()
    samagriStatus: SAMAGRI_STATUS;
}

export class AssignedBookingQuery {
  @ApiProperty()
  @IsString()
    userId: string;
}

export class BookingDetailsQuery {
  @ApiProperty()
  @IsString()
    id: string;
}
