/* eslint-disable prefer-const */
import { replaceAll } from 'src/common/common-utils';
import * as nodemailer from 'nodemailer';
import { Injectable, Logger } from '@nestjs/common';
@Injectable()
export class EmailService {
  logger: Logger;
  props = {
    user: process.env.SMTP_FROM_USER,
    pass: process.env.SMTP_FROM_USER_PASSWORD,
  };

  transporter: any;
  transportOptions: any;
  account: any;

  constructor() {
    this.createTestAccount();
    this.logger = new Logger();
  }

  async createTestAccount() {
    //this.props = await nodemailer.createTestAccount();
    // create reusable transporter object using the default SMTP transport
    this.transportOptions = {
      //host: process.env.SMTP_HOST,
      host: 'smtp.gmail.com',
      type: 'SMTP',
      //port: process.env.SMTP_PORT,
      port: 587,
      //secure: process.env.SMTP_SECURE == "true" ? true : false,
      auth: this.props,
    };
    this.transporter = nodemailer.createTransport(this.transportOptions);
  }

  // transporter.verify((err, success) => {
  //   if (err) console.error(err);
  //   this.logger.log("Your config is correct");
  // });

  sendRawMail = (mailOptions: {
    from: string;
    subject: string;
    html: string;
    to: string[];
    cc: string;
    bcc: string;
    attachments: any | null;
  }) => {
    return new Promise((resolve, reject) => {
      if (typeof mailOptions !== 'object') {
        reject(false);
      } else {
        this.transporter.sendMail(mailOptions, (err: any) => {
          if (err) {
            this.logger.log('Error sending email', err);
          } else {
            resolve('success');
            this.logger.log('Email sent successfully');
          }
        });
      }
    });
  };

  buildEmailData = (
    template: { htmlPart: string; subject: string; dynamicDataKeys: string[] },
    emailDynamicData: any
  ) => {
    let { htmlPart, subject, dynamicDataKeys } = template;
    dynamicDataKeys.forEach((key) => {
      htmlPart = replaceAll(htmlPart, `{{${key}}}`, emailDynamicData[key]);
      subject = replaceAll(subject, `{{${key}}}`, emailDynamicData[key]);
    });
    return {
      htmlBody: htmlPart,
      subject,
    };
  };

  sendMail = async (
    from: 'admin' | 'user',
    emailData: {
      template: {
        htmlPart: string;
        subject: string;
        dynamicDataKeys: string[];
      };
      mailProps: {
        to: string[];
        cc?: string[];
        bcc?: string[];
        attachments?: any[];
      };
      dynamicData: any;
    }
  ) => {
    const htmlCode = this.buildEmailData(
      emailData.template,
      emailData.dynamicData
    );
    return new Promise((resolve, reject) => {
      if (typeof emailData.mailProps !== 'object') {
        reject(false);
      } else {
        const userEmail = process.env.USER_EMAIL ?? 'hello@karishye.co';
        const adminEmail = process.env.ADMIN_EMAIL ?? 'admin@karishye.co';
        const fromEmail = from === 'user' ? userEmail : adminEmail;

        const mailOptions = {
          from: fromEmail,
          subject: htmlCode.subject,
          html: htmlCode.htmlBody,
          to: emailData.mailProps.to,
          cc: emailData.mailProps.cc || [],
          bcc: emailData.mailProps.bcc || [],
          attachments: emailData.mailProps.attachments || [],
        };
        this.logger.log(mailOptions, 'to mail');

        // send email
        this.transporter.sendMail(mailOptions, (err: any) => {
          if (err) {
            this.logger.log('Error sending email', err);
          } else {
            resolve('success');
            this.logger.log('Email sent successfully', '=>', mailOptions.to);
          }
        });
      }
    });
  };
}
