import { Controller, Get, Logger } from '@nestjs/common';
import { ApiResponse, ApiOperation } from '@nestjs/swagger';
import { APIFailedException } from 'src/common/exceptions';
import { PujaProcedureDto } from './models/puja-procedure.dto';
import { PujaProcedureService } from './puja-procedure.service';

interface PujaProcedureResponse {
  status: string;
  data: PujaProcedureDto[];
}
@Controller('puja-procedure')
export class PujaProcedureController {
  logger: Logger;

  constructor(private readonly pujaProcedureService: PujaProcedureService) {
    this.logger = new Logger();
  }

  @Get()
  @ApiOperation({ summary: 'Get all puja procedures' })
  @ApiResponse({
    status: 200,
    description: 'Return all puja procedures',
    type: [PujaProcedureDto],
  })
  async getAllPujaProcedures(): Promise<PujaProcedureResponse> {
    try {
      const procedures = await this.pujaProcedureService.getAllPujaProcedures();

      if (!procedures) {
        throw new APIFailedException('No puja procedures found');
      }

      return {
        status: 'success',
        data: procedures.map((procedure) => ({
          // Map each property from PujaProcedures to PujaProcedureDto
          title: Number(procedure.title), // Assuming title should be a number
          // Add other properties here
        })) as PujaProcedureDto[],
      };
    } catch (error) {
      this.logger.error('Failed to get puja procedures:', error);
      throw new APIFailedException(error);
    }
  }
}
