import { Controller, Get, Logger } from '@nestjs/common';
import { ApiOperation, ApiOkResponse } from '@nestjs/swagger';
import { Samagri } from 'src/entities/samagri';
import { SamagriService } from './samagri.service';

interface SamagriResponse {
  status: string;
  data: Samagri[];
}

@Controller('/')
export class SamagriController {
  logger: Logger;

  constructor(private readonly samagriService: SamagriService) {
    this.logger = new Logger();
  }

  @Get('all-samagri')
  @ApiOperation({ summary: 'To get all the samagri' })
  @ApiOkResponse({ type: [Samagri] })
  async allSamagri(): Promise<SamagriResponse> {
    try {
      const result = await this.samagriService.getAllSamagri();

      if (!result) {
        throw new Error('No samagri found');
      }

      return {
        status: 'success',
        data: result,
      };
    } catch (error) {
      this.logger.error('Failed to get samagri:', error);
      throw error;
    }
  }
}
